#!/bin/bash

prefix="/usr"
presets_dir="$HOME/.config/jack_netmanager/presets"

if [ -z "$prefix" ]; then
	echo -e \
"This file should not be run directly. Otherwise you must specify variables, listed below.
Format:
	jack-netsource-run.sh variable=<value> variable=<value> ...

Mandatory variables (at least one).
	slave_addr
	udp_in
	udp_out
	bind_port (only JACK2)

Optional variables.
	jack_server_name
	jack_client_name

	bitdepth
	celt_bitrate
	samplerate (only JACK1)

	delay
	redundance
	mtu
"
exit; fi

	msg_netsource_missing=$"jack_netsource not found. Do you want to save script for autonomous usage?"
	save_case=$"Save"
	dont_save_case=$"Don't save"
	msg_jack_version=$"You JACK version is not supported.
Choose JACK version you think to be compatible with your JACK"
	msg_no_slave=$"Slave is not specified.
You must specify an IP or domain name, UDP input port, or UDP output port."
	tray_text_search=$"Determining LASH client name"
	tray_text_run=$"launched. Press to icon to termiate it."
	save_dialog_name=$"Save script"
	msg_overwrite=$"Script exsist. What is your actions?."
	replace=$"Replace"
	upgrade=$"Upgrade"
	lash_wrap_missing=$"Unable to use LASH, because \"lash_wrap\" not found"
	msg_already_exist=$"Could not run engine."

netsource_command=jack_netsource

# Checking JACK version. It's necessary for to use right options set.
jack_version=`jackd --version 2>&1 | tail -n 1 | tr ' ' '\n' | head -n 3 | tail -n 1 | tr '.' '\n'`
if   (( `echo "$jack_version" | head -n 1` == 0 )); then jack_version=1
elif (( `echo "$jack_version" | head -n 1` == 1 )); then jack_version=2
else
	jack_version=`zenity --title="" --list --radiolist --text="$msg_jack_version" \
	 --column "" --column "" false 1 false 2`
	if [ -z $jack_version ]; then exit 0; fi
fi

# Switching CLI keys according to jack version
if   [ "$jack_version" = 1 ]; then source "/usr/share/jack-netmanager-gtk/jack-options/jack-1"
elif [ "$jack_version" = 2 ]; then source "/usr/share/jack-netmanager-gtk/jack-options/jack-2"
else exit 0
fi

# If slave is not specified, netsource can't run
slave_specified=0

# If script should be loaded, skip building if command
if [ "$1" != "load" ]
then source /usr/share/jack-netmanager-gtk/parse_gui_settings; fi

# Enable lash support
if [ "$use_lash_button" = "true" ]; then
if [ -n "`which lash_wrap`" ]; then
	if [ -n "$jack_client_name" ]; then						# echo "Specified JACK client name: $jack_client_name"
	# Checking if this name is not`pgrep -n -U \`whoami\` lash_wrap` owned
		found_names="`echo list | lash_control`"				#; echo "Found names: $found_names" | less
		names_pos="`echo   \"$found_names\" | sgrep -o "%e" '"clients:\n"'`"	#; echo "names pos:  $names_pos"    | less
		found_names="`echo \"$found_names\" | tr '\n' ':' | cut -c \"$names_pos-\" | tr ':' '\n' | tr -s '\n'`"
											  #echo "Found names: $found_names" | less
		found_names="`echo \"$found_names\" | grep \"$jack_client_name\"`"	#; echo "Found names: $found_names" | less

		# Also this name must not be owned at JACK namespace
		found_names="$found_names"`jack_lsp | sgrep -o "%r" "\"\nnew-jack-network-01:\""`
											#echo "Found names+JACK: $found_names"

		# If owned, use lash_wrap to take it from jack name
		if [ -n "$found_names" ]; then			# echo "Specified name is registered in LASH or JACK. Determining JACK name."
			probe_client_name="$jack_name_opt\"$jack_client_name\""
			unset jack_client_name
		fi
	fi
	# If jack client name is not specified, take it from jack via lash_wrap probe.
	if [ -z "$jack_client_name" ]; then echo $"JACK name is scrambled or not specified. LASH will take it from JACK."
		zenity --window-icon="/usr/share/pixmaps/jack-netmanager-gtk/name-search.svg" \
		       --notification --text="$tray_text_search" & search_pid=$!
		lash_probe=`bash -c "lash_wrap -p -- $netsource_command $probe_client_name" | grep "* lash_wrap"`
		kill -15 $search_pid							#; echo "lash probe: $lash_probe"

		option_line=`echo "$lash_probe" | sgrep -o "%s" '"-j"'`			#; echo "option line: $option_line"
		jack_client_name=`echo   "$lash_probe" | cut -c "$option_line-" \
		| tr '"' '\n' | head -n 2 | tail -n 1`					#; echo "lash name: $jack_client_name"
	fi
	lash_command="lash_wrap -j \"$jack_client_name\" -l \"$jack_client_name\""
	netsource_command="$lash_command -- $netsource_command"
else
	echo "$lash_wrap_missing"
	zenity --error --text="$lash_wrap_missing"
fi fi

# LADI Session Handler support

# If client name still is not specified, supply default name to form tray text.
if [ -n "$jack_client_name" ]; then
	netsource_command="$netsource_command $jack_name_opt \"$jack_client_name\""
else
	jack_client_name="JACK Network Source $((`cat $HOME/.jack-netmanager-gtk.proc | wc -l`+1))"
fi

# Enable tray icon in exported scripts
#netsource_command="$netsource_command"

## BREAKPOINT
#echo "netsource command: $netsource_command"; exit 0

#==================================================================
# Arguments parsed, time to launch

if [ "$1" == "load" ]; then
	cd "$presets_dir"
	preset="`zenity --file-selection --title=$"Load preset"`"
	if ! [ -f "$preset" ]; then exit 0; fi
	netsource_command=`cat "$preset"`
fi

if [ "$1" == "run" -o "$1" == "load" ]; then
#echo "Said to run."
# Checking, if jack_netsource present.
 if [ -z "`which jack_netsource`" ]; then
	echo $msg_netsource_missing
	zenity --error --text="$msg_netsource_missing"
	exit 0
 fi

 if [ "$netsource_missing" != "true" ]; then

	# If LADI Session Handler is requested, simply pass command to ladish and exit
	if [ "$use_ladish_button" == "true" ]; then
		echo "$netsource_command"

		if [ -n "$ladish_room" ]
			then ladish_control rnewapp "$ladish_room" "$netsource_command" "$jack_client_name"
			else ladish_control snewapp "$netsource_command" "$jack_client_name"
		fi
		exit 0
	fi

	# Otherwise.....
	bash -c "$netsource_command" &
	netsource_pid=$!
	pgrep -P $netsource_pid

	# Checking, that found process is new
	msg_already_exist="$msg_already_exist"=$netsource_pid
	if [ -e "$HOME/.jack-netmanager-gtk.proc" ]; then
		if [ -n "`cat ~/.jack-netmanager-gtk.proc | grep $netsource_pid`" ]; then
			echo $msg_already_exist
			zenity --error --text="$msg_already_exist"
			exit 0
		fi
	fi

	# Registering our process in per-user list
	echo $netsource_pid >> ~/.jack-netmanager-gtk.proc

	# Launch tray icon.
	zenity --window-icon="/usr/share/pixmaps/jack-netmanager-gtk/window-icon.svg" --notification \
	       --text="$jack_client_name" &
	handler_pid=$!

	while ([ -z "$quit" ]); do
		if [ -z "`ps -p $netsource_pid -o pid --no-heading`" ]; then kill -15 $handler_pid; quit=yes; fi
		if [ -z "`ps -p $handler_pid -o pid --no-heading`" ]; then kill -15 $netsource_pid; quit=yes; fi
		sleep 0.5
	done

	# Unregistering killed process from per-user list
	cat ~/.jack-netmanager-gtk.proc | grep -v $netsource_pid | tr -s '\n' > ~/.jack-netmanager-gtk.proc.tmp
	cat ~/.jack-netmanager-gtk.proc.tmp > ~/.jack-netmanager-gtk.proc;
	rm ~/.jack-netmanager-gtk.proc.tmp
 fi
 exit 0
fi

# Save script for autonomous usage
if [ "$1" == "save" ]; then
	# echo "Said to save."
	filename="$presets_dir/"`zenity --entry --title="$save_dialog_name" --text="" --entry-text="$jack_client_name"`
	if [ -n "$filename" ]; then
		# Checking, wether file already exist
		if [ -e "$filename" ]; then
			if [ -z "`zenity --title="" --question --text=\"$msg_overwrite\"`" ]
			then exit 0; fi
		else
		# If not, directory also may absent
			if ! [ -e "$presets_dir" ]; then mkdir --parents "$presets_dir"; fi
		fi

		echo "$netsource_command" > "$filename"
	fi
	exit 0
fi

exit 0
